// Copyright (c) 2007 David Grangier
// Copyright (c) 2007 Samy Bengio
// 
// All rights reserved.
// 
// Redistribution and use in source and binary forms, with or without 
// modification, are permitted provided that the following conditions are 
// met: Redistributions of source code must retain the above copyright 
// notice, this list of conditions and the following disclaimer.
// Redistributions in binary form must reproduce the above copyright
// notice, this list of conditions and the following disclaimer in the 
// documentation and/or other materials provided with the distribution.
// The name of the author may not be used to endorse or promote products
// derived from this software without specific prior written permission.
// 
// THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR 
// IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, 
// INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR 
// SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) 
// HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
// STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
// ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
// THE POSSIBILITY OF SUCH DAMAGE.


#include "ImgConstraintPolicies.h"
#include "ImgConstraintDataset.h"
#include "SVectorCalculator.h"

namespace Torch {

// ----------------------------------------------------------------
// Text Epsilon Margin
// ----------------------------------------------------------------
TextEpsilon::TextEpsilon(real min_epsilon_, SparseMatrix *captions_)
{
	min_epsilon = min_epsilon_;
	captions = captions_;
}

real TextEpsilon::getEpsilon(ImgConstraintDataset* d)
{
	svector *que = d->queries->lines + (d->cur_que);
	svector *pos = captions->lines + (d->cur_pos);	
	svector *neg = captions->lines + (d->cur_neg);
	real score = SVectorCalculator::inner(que, pos);
	score -= SVectorCalculator::inner(que, neg);
	if (score < min_epsilon) score = min_epsilon;

	return score;
}

TextEpsilon::~TextEpsilon()
{}

// -----------------------------------------------------------------
// Break-Even Point Weighting
// -----------------------------------------------------------------
BEPWeighting::BEPWeighting() 
{}

real BEPWeighting::getWeight(ImgConstraintDataset* d)
{
	int q = d->cur_que;
	return 1.0/((real)(d->relevance->lines[q].size));
}

BEPWeighting::~BEPWeighting()
{}

}
